import { AbstractProperty } from "../properties/abstract-property";
import { RuleEngineUpdateHandler } from "./rule-engine-update-handler-impl";
import { Snapshot } from "./snapshot/snapshot";
import { BuilderOptions } from "./builder/builder-options";
import { Builder } from "./builder/builder";
import { ValidationResult } from "../validators/validation-result";
import { ValidatorInstance } from "./validation/validator-instance-impl";
import { AbstractDataProperty } from "../properties/abstract-data-property";
import { PropertyGroup } from "../properties/group-of-properties";
import { Module } from "./modules/module";
export declare class RuleEngine implements RuleEngineUpdateHandler {
    private readonly builder;
    private readonly propertyMap;
    private readonly dependencyGraph;
    private readonly validations;
    private readonly dataLinks;
    private readonly snapshots;
    private propertiesThatRequireAnEagerUpdate;
    private get properties();
    constructor(options: BuilderOptions);
    defineModule<S extends PropertyGroup>(initFcn: (builder: Builder) => S): Module<S>;
    getBuilder(): Builder;
    getPropertyById(id: string): AbstractProperty | undefined;
    takeSnapShot(key?: string): Snapshot;
    restoreSnapShot(key?: string): void;
    linkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    unlinkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    private createDataLinkKey;
    needsAnUpdate(mightHaveChanged: AbstractProperty): void;
    updateValue(property: AbstractProperty): Promise<void>;
    private hasBeenUpdated;
    invalidateValidationResults(validators: readonly ValidatorInstance<readonly AbstractProperty[]>[]): void;
    validate(validators: readonly ValidatorInstance<readonly AbstractProperty[]>[]): Promise<ValidationResult>[];
    private getValidationProcess;
}
